@@ -30,18 +30,6 @@
 #pod It is unlikely that someone else will want to use it, so I will not bother with documenting it, at
 #pod least for now.
 #pod
-#pod =option minimum_perl
-#pod
-#pod =option fake_release
-#pod
-#pod If true (C<1>), release will be a fake one, i. e. no external operations will be done:
-#pod C<UploadToCPAN> plugin will be replaced with C<FakeRelease> (which does nothing), C<ArchiveRelease>
-#pod will be skipped, and C<hg push> will not be performed.
-#pod
-#pod Option can be set trough F<dist.ini> file or with C<FAKE_RELEASE> environment variable.
-#pod
-#pod C<Bool>, optional, default value is C<0>.
-#pod
 #pod =for Pod::Coverage configure
 #pod
 #pod =cut
@@ -50,61 +38,70 @@ package Dist::Zilla::PluginBundle::Author::VDB;
 
 use Moose;
 use namespace::autoclean;
-use version 0.77;
 
 # PODNAME: Dist::Zilla::PluginBundle::Author::VDB
 # ABSTRACT: VDB's plugin bundle
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.006'; # VERSION
 
 with 'Dist::Zilla::Role::PluginBundle::Easy';
 
+use Carp qw{ croak };
+
 #   Require all the modules explicitly now to avoid unexpected failures in the middle of release.
 use IPC::System::Simple ();
-use Path::Tiny          ();
+use Path::Tiny 0.070;   ();
 use Perl::Version       ();
+use File::chdir         ();
 
 #   Explicitly require all the plugins to let them appear in the metadata.
-use Dist::Zilla::Plugin::Hook                       0.005   ();   # We need Hook::PrereqSource.
-use Dist::Zilla::Plugin::GatherFromManifest                 ();
-use Dist::Zilla::Plugin::Manifest::Write                    ();
-use Dist::Zilla::Plugin::OurPkgVersion                      ();
-use Dist::Zilla::Plugin::SurgicalPodWeaver                  ();
-use Dist::Zilla::Plugin::Test::EOL                          ();
-use Dist::Zilla::Plugin::Test::NoTabs                       ();
-use Dist::Zilla::Plugin::MojibakeTests                      ();
-use Dist::Zilla::Plugin::Test::Compile                      ();
-use Dist::Zilla::Plugin::Test::Version                      ();
-use Dist::Zilla::Plugin::Test::NewVersion                   ();
-use Dist::Zilla::Plugin::Test::MinimumVersion               ();
-use Dist::Zilla::Plugin::Test::Perl::Critic                 ();
-use Dist::Zilla::Plugin::PodSyntaxTests                     ();
-use Dist::Zilla::Plugin::PodCoverageTests                   ();
-use Dist::Zilla::Plugin::Test::PodSpelling                  ();
-use Dist::Zilla::Plugin::Test::Pod::No404s                  ();
-use Dist::Zilla::Plugin::MetaTests                          ();
-use Dist::Zilla::Plugin::Test::CPAN::Meta::JSON             ();
-use Dist::Zilla::Plugin::Test::CPAN::Changes                ();
-use Dist::Zilla::Plugin::Test::DistManifest                 ();
-use Dist::Zilla::Plugin::Test::Kwalitee                     ();
-use Dist::Zilla::Plugin::MinimumPerl                        ();
 use Dist::Zilla::Plugin::AutoPrereqs                        ();
-use Dist::Zilla::Plugin::Prereqs::AuthorDeps                ();
-use Dist::Zilla::Plugin::CheckPrereqsIndexed                ();
-use Dist::Zilla::Plugin::MetaProvides::Package              ();
-use Dist::Zilla::Plugin::MetaResources::Template            ();
-use Dist::Zilla::Plugin::MetaYAML                           ();
-use Dist::Zilla::Plugin::MetaJSON                           ();
-use Dist::Zilla::Plugin::ModuleBuildTiny                    ();
-use Dist::Zilla::Plugin::CheckExtraTests                    ();
 use Dist::Zilla::Plugin::CheckChangesHasContent             ();
-use Dist::Zilla::Plugin::TestRelease                        ();
+use Dist::Zilla::Plugin::CheckExtraTests                    ();
+use Dist::Zilla::Plugin::CheckPrereqsIndexed                ();
 use Dist::Zilla::Plugin::ConfirmRelease                     ();
 use Dist::Zilla::Plugin::FakeRelease                        ();
-use Dist::Zilla::Plugin::UploadToCPAN                       ();
+use Dist::Zilla::Plugin::GatherFromManifest                 ();
+use Dist::Zilla::Plugin::Hook                       0.007   ();   # We need Hook::Releaser.
+use Dist::Zilla::Plugin::Manifest::Write                    ();
+use Dist::Zilla::Plugin::MetaJSON                           ();
+use Dist::Zilla::Plugin::MetaProvides::Package              ();
+use Dist::Zilla::Plugin::MetaResources::Template            ();
+use Dist::Zilla::Plugin::MetaTests                          ();
+use Dist::Zilla::Plugin::MetaYAML                           ();
+use Dist::Zilla::Plugin::MinimumPerl                        ();
+use Dist::Zilla::Plugin::ModuleBuildTiny                    ();
+use Dist::Zilla::Plugin::MojibakeTests                      ();
 use Dist::Zilla::Plugin::NextRelease                        ();
+use Dist::Zilla::Plugin::OurPkgVersion                      ();
+use Dist::Zilla::Plugin::PodCoverageTests                   ();
+use Dist::Zilla::Plugin::PodSyntaxTests                     ();
+use Dist::Zilla::Plugin::Prereqs::AuthorDeps                ();
+use Dist::Zilla::Plugin::SurgicalPodWeaver                  ();
+use Dist::Zilla::Plugin::Templates                          ();
+use Dist::Zilla::Plugin::Test::Compile                      ();
+use Dist::Zilla::Plugin::Test::CPAN::Changes                ();
+use Dist::Zilla::Plugin::Test::CPAN::Meta::JSON             ();
+use Dist::Zilla::Plugin::Test::DistManifest                 ();
+use Dist::Zilla::Plugin::Test::EOL                          ();
+use Dist::Zilla::Plugin::Test::Kwalitee                     ();
+use Dist::Zilla::Plugin::Test::MinimumVersion               ();
+use Dist::Zilla::Plugin::Test::NewVersion                   ();
+use Dist::Zilla::Plugin::Test::NoTabs                       ();
+use Dist::Zilla::Plugin::Test::Perl::Critic                 ();
+use Dist::Zilla::Plugin::Test::Pod::No404s                  ();
+use Dist::Zilla::Plugin::Test::PodSpelling                  ();
+use Dist::Zilla::Plugin::TestRelease                        ();
+use Dist::Zilla::Plugin::Test::Version                      ();
+use Dist::Zilla::Plugin::UploadToCPAN                       ();
 
 #   One more implicitly required module.
-use Pod::Weaver::PluginBundle::Author::VDB ();
+use Pod::Weaver::PluginBundle::Author::VDB                  ();
+
+#pod =option minimum_perl
+#pod
+#pod C<Str>, optional, default value 5.006.
+#pod
+#pod =cut
 
 has minimum_perl => (
     is          => 'ro',
@@ -116,6 +113,18 @@ has minimum_perl => (
     },
 );
 
+#pod =option fake_release
+#pod
+#pod If true (C<1>), release will be a fake one, i. e. no external operations will be done:
+#pod C<UploadToCPAN> plugin will be replaced with C<FakeRelease> (which does nothing), and C<hg push>
+#pod will not be performed.
+#pod
+#pod Option can be set trough F<dist.ini> file or with C<FAKE_RELEASE> environment variable.
+#pod
+#pod C<Bool>, optional, default value is 0.
+#pod
+#pod =cut
+
 has fake_release => (
     is          => 'ro',
     isa         => 'Bool',
@@ -126,13 +135,88 @@ has fake_release => (
     },
 );
 
+#pod =option archive
+#pod
+#pod Directory to archive files to. If empty, release will not be archived. If such directory does not
+#pod exist, it will be created before release.
+#pod
+#pod C<Str>, optional, default value ".releases".
+#pod
+#pod =cut
+
+has archive => (
+    is          => 'ro',
+    isa         => 'Str',
+    lazy        => 1,
+    default     => sub {
+        my ( $self ) = @_;
+        return $self->payload->{ archive } // '.releases';
+    },
+);
+
+#pod =option templates
+#pod
+#pod This option will be passed to C<Templates> plugin. If you no not want C<Templates> to process
+#pod files, specify C<:NoFiles>. This is multi-value option (i. e. may be specified several times).
+#pod
+#pod C<ArrayRef[Str]>, optional, default value C<[ ':InstallModules' ]>.
+#pod
+#pod =cut
+
+has templates => (
+    is          => 'ro',
+    isa         => 'ArrayRef[Str]',
+    lazy        => 1,
+    default     => sub {
+        my ( $self ) = @_;
+        return $self->payload->{ templates } // [ ':InstallModules' ];
+    },
+);
+
+#pod =option unwanted_module
+#pod
+#pod TODO
+#pod
+#pod =cut
+
+has unwanted_modules => (
+    is          => 'ro',
+    isa         => 'Str',
+    lazy        => 1,
+    default     => sub {
+        my ( $self ) = @_;
+        my $p  = $self->payload;
+        my $u1 = $p->{ unwanted_module  };
+        my $u2 = $p->{ unwanted_modules };
+        my $u;
+        if ( $u1 or $u2 ) {
+            $u = join( ' ', $u1 ? @$u1 : (), $u2 ? @$u2 : () );
+        } else {
+            $u = 'DDP Data::Printer';
+        };
+        if ( $u =~ m{[{,}]}x ) {
+            croak "oops";
+        };
+        return "qw{ $u }";
+    },
+);
+
+#pod =method mvp_multivalue_args
+#pod
+#pod =cut
+
+sub mvp_multivalue_args {
+    return qw{ templates unwanted_module unwanted_modules };
+};
+
 sub configure {
 
     my ( $self ) = @_;
 
     $self->add_plugins(
 
-        [ 'Hook' => 'prologue' => { # DOES NOT WORK.
+        [ 'Hook' => 'prologue' => {
+            # DOES NOT WORK because plugin name will be '@Author::VDB/prologue'.
             'hook' => [ q{
                 use autodie ':all';
                 use IPC::System::Simple qw{ capture };
@@ -143,7 +227,14 @@ sub configure {
         [ 'Hook::VersionProvider' => {
             'hook' => [ q{
                 use Path::Tiny;
-                path( 'VERSION' )->slurp =~ s{\s+\z}{}r;
+                my $file = path( 'VERSION' );
+                my $version = $file->slurp =~ s{\s+\z}{}r;
+                if ( $version !~ m{\A@{ [ Perl::Version::REGEX ] }\z} ) {
+                    $self->abort( [
+                        'Invalid version string "%s" at %s line 1', $version, "$file"
+                    ] );
+                };
+                $version;
             } ],
         } ],
 
@@ -195,11 +286,11 @@ sub configure {
         [ 'OurPkgVersion' ],
 
         [ 'SurgicalPodWeaver' => {
-            'replacer'      => 'replace_with_comment',
             'config_plugin' => '@Author::VDB',
+            'replacer'      => 'replace_with_comment',
         } ],
 
-        [ 'FileFinder::ByName' => 'doc' => {
+        [ 'FileFinder::ByName' => 'source doc' => {
             # Plugin name will be `@Author::VDB/doc`.
             'file' => [
                 'COPYING',
@@ -209,7 +300,8 @@ sub configure {
 
         [ 'Templates' => {
             'templates' => [
-                '@Author::VDB/doc',
+                '@Author::VDB/source doc',
+                @{ $self->templates },
             ],
         } ],
 
@@ -221,7 +313,7 @@ sub configure {
         [ 'Hook::AfterBuild' => {
             'hook' => [ q{
                 use Path::Tiny;
-                my $files = $zilla->plugin_named( '@Author::VDB/doc' )->find_files();
+                my $files = $zilla->plugin_named( '@Author::VDB/source doc' )->find_files();
                 my $build = path( $arg->{ build_root } );
                 my $root  = path( $dist->root );
                 for my $file ( @$files ) {
@@ -283,15 +375,15 @@ sub configure {
 
         [ 'Test::PodSpelling' => {
             'spell_cmd' => 'aspell list -l en -p ./xt/aspell.en.pws',
-                #   Leading dot (in `./xt/aspell.en.pws`) is important! Whitout the dot `aspell` fails to
-                #   find the dictionary.
+                #   Leading dot (in `./xt/aspell.en.pws`) is important! Whitout the dot `aspell`
+                #    fails to find the dictionary.
         } ],
 
         [ 'Test::Pod::No404s' ],    # No dead URLs.
 
         # Metadata
 
-        [ 'MetaTests' ],                # `Dist-Zilla`-bundled test, uses `Test::CPAN::Meta`, checks `META.yml`.
+        [ 'MetaTests' ],    # `Dist-Zilla`-bundled test, uses `Test::CPAN::Meta`, checks `META.yml`.
 
         [ 'Test::CPAN::Meta::JSON' ],   # Uses `Test::CPAN::Meta::JSON`.
 
@@ -404,11 +496,31 @@ sub configure {
         #   Release
         #
 
+        $self->archive ? (
+            #   Make sure archive directory exists. Do it in the very beginnig of release because
+            #   it is simple and fast operation. It will be annoying to pass all the tests and
+            #   stop release  because `dzil` fails to create archive directory.
+            [ 'Hook::BeforeRelease' => 'archive directory' => {
+                'hook' => [ q{
+                    my $root = path( $self->zilla->root . '' );
+                    my $dir = path( "} . $self->archive . q{" );
+                    if ( $dir->is_absolute ) {
+                        $self->log_error( [ 'Bad archive directory: %s', "$dir" ] );
+                        $self->log_error( [ 'Absolute path not allowed.' ] );
+                        $self->abort();
+                    };
+                    if ( not $dir->is_dir ) {
+                        $self->log( [ 'Creating archive directory %s...', "$dir" ] );
+                        $dir->mkpath();
+                    };
+                } ],
+            } ]
+        ) : (
+        ),
+
         #~ [ 'RunExtraTests' ],         # Does not work for me, see <https://github.com/dagolden/Dist-Zilla-Plugin-CheckExtraTests/issues/26>.
         [ 'CheckExtraTests' ],          # So use CheckExtraTest meanwhile.
 
-        [ 'CheckChangesHasContent' ],
-
         [ 'TestRelease'     ],          # Unpack tarball and run tests.
 
         #   Make sure the distro does not depend on unwanted modules. Unwanted module, for example,
@@ -416,12 +528,17 @@ sub configure {
         #   debugging code.
         [ 'Hook::BeforeRelease' => 'unwanted deps' => {
             'hook' => [ q{
-                my @modules = qw{ DDP Data::Printer };          # TODO: Parameter
+                my @modules = } . $self->unwanted_modules . q{;
                 my $prereqs = $dist->distmeta->{ prereqs };
+                my $header;
                 for my $module ( @modules ) {
                     for my $stage ( qw{ configure develop runtime test } ) {
                         if ( exists( $prereqs->{ $stage }->{ requires }->{ $module } ) ) {
-                            $self->log_error( [ '%s found in %s prereqs ', $module, $stage ] );
+                            if ( not $header ) {
+                                $self->log_error( 'Unwanted modules found:' );
+                                $header = 1;
+                            };
+                            $self->log_error( [ '    %s in %s prereqs ', $module, $stage ] );
                         };
                     };
                 };
@@ -429,6 +546,18 @@ sub configure {
             } ]
         } ],
 
+        [ 'CheckChangesHasContent' ],
+
+        [ 'Hook::BeforeRelease' => 'hg check tag' => {
+            'hook' => [ q{
+                #   Make sure there is no such tag yet.
+                use Path::Tiny;
+                if ( path( '.hgtags' )->slurp_utf8 =~ m{ \Q$MY::version\E$}m ) {
+                    $self->abort( [ 'tag %s already exists', $MY::version ] );
+                };
+            } ],
+        } ],
+
         [ 'Hook::BeforeRelease' => 'hg stat' => {
             'hook' => [ q{
                 #   Make sure all the files committed.
@@ -443,19 +572,34 @@ sub configure {
             } ],
         } ],
 
-        [ 'Hook::BeforeRelease' => 'hg check tag' => {
+        [ 'ConfirmRelease' ],       # Ask confirmation before uploading the release.
+
+        [ 'Hook::Releaser' => 'tgz' => {
             'hook' => [ q{
-                #   Make sure there is no such tag yet.
-                use Path::Tiny;
-                if ( path( '.hgtags' )->slurp_utf8 =~ m{ \Q$MY::version\E$}m ) {
-                    $self->abort( [ 'tag %s already exists', $MY::version ] );
-                };
+                $MY::tgz = $arg;
             } ],
         } ],
 
-        [ 'ConfirmRelease' ],       # Ask confirmation before uploading the release.
+        #   `Archive` is a good plugin, but I want to copy, not move tarball, because I am going to
+        #   install it later.
+        $self->archive ? (
+            [ 'Hook::Releaser' => 'archive release' => {
+                'hook' => [ q{
+                    use Path::Tiny;
+                    my $tgz = path( $arg );
+                    my $dir = path( "} . $self->archive . q{" );
+                    $self->log( [ 'Copying %s to %s...', "$tgz", "$dir" ] );
+                    $tgz->copy( $dir->child( $tgz->basename ) );
+                } ],
+            } ],
+        ) : (
+        ),
 
-        [ $self->fake_release ? 'FakeRelease' : 'UploadToCPAN' ],
+        $self->fake_release ? (
+            [ 'FakeRelease' ],
+        ) : (
+            [ 'UploadToCPAN' ],
+        ),
 
         [ 'Hook::AfterRelease' => 'hg add tag' => {
             'hook' => [ q{
@@ -475,13 +619,6 @@ sub configure {
             } ],
         } ],
 
-        $self->fake_release ? (
-        ) : (
-            [ 'ArchiveRelease' => {
-                'directory'   => '.releases',
-            } ],
-        ),
-
         [ 'NextRelease' => {
             'format'      => '%V @ %{yyyy-MM-dd HH:mm zzz}d',
             'time_zone'   => 'UTC',
@@ -521,6 +658,17 @@ sub configure {
             ],
         } ],
 
+        [ 'Hook::AfterRelease' => 'install' => {
+            'hook' => [ q{
+                use autodie ':all';
+                use File::chdir;
+                local $CWD = $zilla->built_in;
+                system( 'cpanm', '--notest', '.' );
+                    # ^ We run the tests on unpacked tarball before release, no need in running
+                    #   tests one more time.
+            } ],
+        } ],
+
         [ 'Hook::AfterRelease' => 'clean' => {
             'hook' => [ q{
                 $zilla->clean();
@@ -577,7 +725,7 @@ Dist::Zilla::PluginBundle::Author::VDB - VDB's plugin bundle
 
 =head1 VERSION
 
-Version 0.005, released on 2015-08-11 20:08 UTC.
+Version 0.006, released on 2015-08-14 00:31 UTC.
 
 =head1 WHAT?
 
@@ -588,19 +736,43 @@ C<Dist-Zilla-PluginBundle-Author-VDB> (or just C<@Author::VDB>) is a C<Dist-Zill
 It is unlikely that someone else will want to use it, so I will not bother with documenting it, at
 least for now.
 
+=head1 OBJECT METHODS
+
+=head2 mvp_multivalue_args
+
 =head1 OPTIONS
 
 =head2 minimum_perl
 
+C<Str>, optional, default value 5.006.
+
 =head2 fake_release
 
 If true (C<1>), release will be a fake one, i. e. no external operations will be done:
-C<UploadToCPAN> plugin will be replaced with C<FakeRelease> (which does nothing), C<ArchiveRelease>
-will be skipped, and C<hg push> will not be performed.
+C<UploadToCPAN> plugin will be replaced with C<FakeRelease> (which does nothing), and C<hg push>
+will not be performed.
 
 Option can be set trough F<dist.ini> file or with C<FAKE_RELEASE> environment variable.
 
-C<Bool>, optional, default value is C<0>.
+C<Bool>, optional, default value is 0.
+
+=head2 archive
+
+Directory to archive files to. If empty, release will not be archived. If such directory does not
+exist, it will be created before release.
+
+C<Str>, optional, default value ".releases".
+
+=head2 templates
+
+This option will be passed to C<Templates> plugin. If you no not want C<Templates> to process
+files, specify C<:NoFiles>. This is multi-value option (i. e. may be specified several times).
+
+C<ArrayRef[Str]>, optional, default value C<[ ':InstallModules' ]>.
+
+=head2 unwanted_module
+
+TODO
 
 =head1 WHY?
 
